from utilities.dcmgeometrysdk.landxml.landxml import CgPoint
from utilities.dcmgeometrysdk.landxml.landxml import RedHorizontalPosition, RedVerticalObservation
from utilities.dcmgeometrysdk.geometryfunctions.bearingdistancefunctions import calc_distance
import shapely.geometry as sg
from copy import deepcopy


# class ReducedHoriztonalPosition:
#     def __init__(self, reduced_hoz_pos=None):
#         self.desc = None
#         self.name = None
#         self.oid = None
#         self.setup_id = None
#         self.date = None
#         self.horizontal_datum = None
#         self.horizontal_adjustment = None
#         self.latitude = None
#         self.longitude = None
#         self.horizontal_fix = None
#         self.currency_date = None
#         self.order = None
#         self.positional_uncertainty = None
#         if reduced_hoz_pos is not None:
#             self.set_red_hoz_pos(reduced_hoz_pos)
#
#     def set_red_hoz_pos(self, red_hoz_pos):
#         red_hoz_pos: RedHorizontalPosition
#         self.oid = red_hoz_pos.oID
#         self.desc = red_hoz_pos.desc
#         self.name = red_hoz_pos.name
#         self.setup_id = red_hoz_pos.setupID
#         self.date = red_hoz_pos.date
#         self.horizontal_datum = red_hoz_pos.horizontalDatum
#         self.horizontal_adjustment = red_hoz_pos.horizontalAdjustment
#         self.latitude = red_hoz_pos.latitude
#         self.longitude = red_hoz_pos.longitude
#         self.horizontal_fix = red_hoz_pos.horizontalFix
#         self.currency_date = red_hoz_pos.currencyDate
#         self.order = red_hoz_pos.order
#         self.positional_uncertainty = red_hoz_pos.positionalUncertainity

class PointGeom:
    """Class to hold Point Geometry, will hold original geometry and any updated geometries
     need to think about how to hold all different situations in here"""
    def __init__(self, point=None, monument=None, reduced_hoz_pos=None, reduced_vert_obs=None):
        self.original_crs = None
        self.crs = None
        self.mon_state = None
        self.mon_type = None
        self.mon_condition = None
        self.mon_desc = None
        self.mon_origin_survey = None
        self.horizontal_position = RedHorizontalPosition()
        self.vertical_position = RedVerticalObservation()
        self.translated = False
        self.translated_x = 0
        self.translated_y = 0
        self.translated_z = 0
        self.ccc = False
        self.p_constrained = -1
        self.q_constrained = -1
        self.height = None
        if isinstance(point, CgPoint):
            self.name = point.name
            self.point_type = point.pntSurv
            self.point_oid = point.oID
            self.associated_point_oid = None
            self.associated_point_distance = None
            self.point_state = point.state
            self.latitude = point.latitude
            self.longitude = point.longitude
            self.neh = point.valueOf_.split()
            if len(self.neh) > 2:
                self.height = self.neh[2]
            self.original_geom = self.get_original_geom()
            self.geometry = deepcopy(self.original_geom)
            if monument is not None:
                self.set_monument(monument)
            if reduced_hoz_pos is not None:
                self.horizontal_position = reduced_hoz_pos
            if reduced_vert_obs is not None:
                self.vertical_obs = reduced_vert_obs
            
        else:
            self.name = None
            self.point_type = None
            self.point_oid = None
            self.associated_point_oid = None
            self.associated_point_distance = None
            self.point_state = None
            self.latitude = None
            self.longitude = None
            self.neh = None
            self.original_geom = None
            self.geometry = None

        if self.name is None:
            pass
        elif not isinstance(self.name, str):
            self.name = str(self.name)

    def set_monument(self, monument):
        self.mon_state = monument.state
        self.mon_type = monument.type_
        self.mon_condition = monument.condition
        self.mon_desc = monument.desc
        self.mon_origin_survey = monument.originSurvey


    def get_original_geom(self):
        neh = self.neh
        if None in neh:
            neh = [self.latitude, self.longitude]
            if len(self.neh) == 3:
                neh += [self.neh[2]]
        if len(neh) == 3:
            enh = [neh[1]] + [neh[0] + neh[2]]
        else:
            enh = [neh[1]] + [neh[0]]
        return sg.Point((float(x) for x in enh))

    def set_new_geometry(self, geometry):
        if isinstance(geometry, sg.Point):
            if self.original_geom is None:
                if self.geometry is not None:
                    self.original_geom = deepcopy(self.geometry)
                else:
                    self.original_geom = geometry
            self.geometry = geometry
        else:
            raise TypeError()

    def set_associated_id(self, oid, distance):
        self.associated_point_oid = oid
        self.associated_point_distance = distance

    def get_distance_between_orig_new_coords(self):
        return calc_distance(self.original_geom, self.geometry)

    def round_geometry(self, decimals=None):
        geom = sg.Point((round(self.geometry.x, decimals), round(self.geometry.y, decimals)))
        self.set_new_geometry(geom)

    def get_long_lat(self):
        x = self.geometry.x
        y = self.geometry.y
        return x, y
